/*
	Pioneer PX-7 Emulator 'ePX-7'
	ASCII MSX2 with DEMPA Sensor Kid Emulator 'yayaMSX2SK'

	Author : Takeda.Toshiya
	Date   : 2014.01.09-

	modified by umaiboux

	[ memory ]
*/

#ifndef _MEMORY_H_
#define _MEMORY_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"

#define SIG_LDC_EXV	0
#define SIG_LDC_ACK	1
#define SIG_LDC_MUTE	2

#define SIG_MEMORY_SEL	0

class SLOT_ROMRAM32K : public DEVICE
{
private:
	uint8 wdmy[0x2000];
	uint8 rdmy[0x2000];
	uint8* wbank[8];
	uint8* rbank[8];
	uint8 rom[0x8000];
	uint8 ram[0x8000];
	
public:
	SLOT_ROMRAM32K(VM* parent_vm, EMU* parent_emu) : DEVICE(parent_vm, parent_emu) {}
	~SLOT_ROMRAM32K() {}
	
	// common functions
	void initialize();
	void write_data8(uint32 addr, uint32 data);
	uint32 read_data8(uint32 addr);
};

#define MAX_TRACKS 1024

class SLOT_LDC : public DEVICE
{
private:
	DEVICE *d_cpu, *d_ldp, *d_vdp;
	
	uint8 wdmy[0x2000];
	uint8 rdmy[0x2000];
	uint8* wbank[8];
	uint8* rbank[8];
	uint8 rom[0x2000];
	
	bool clock;
	bool exv, ack;
	bool super_impose;
	bool req_intr;
	bool pc4, mute_l, mute_r;
	
public:
	SLOT_LDC(VM* parent_vm, EMU* parent_emu) : DEVICE(parent_vm, parent_emu) {}
	~SLOT_LDC() {}
	
	// common functions
	void initialize();
	void reset();
	void write_data8(uint32 addr, uint32 data);
	uint32 read_data8(uint32 addr);
	void write_signal(int id, uint32 data, uint32 mask);
	void event_callback(int event_id, int err);
	
	// unique function
	void set_context_cpu(DEVICE* device)
	{
		d_cpu = device;
	}
	void set_context_ldp(DEVICE* device)
	{
		d_ldp = device;
	}
	void set_context_vdp(DEVICE* device)
	{
		d_vdp = device;
	}
};

class SLOT_CART : public DEVICE
{
private:
	uint8 wdmy[0x2000];
	uint8 rdmy[0x2000];
	uint8* wbank[8];
	uint8* rbank[8];
	uint8 rom[0x10000];
	
	bool inserted;
	
public:
	SLOT_CART(VM* parent_vm, EMU* parent_emu) : DEVICE(parent_vm, parent_emu) {}
	~SLOT_CART() {}
	
	// common functions
	void initialize();
	void write_data8(uint32 addr, uint32 data);
	uint32 read_data8(uint32 addr);
	
	// unique functions
	void open_cart(_TCHAR *file_path);
	void close_cart();
	bool cart_inserted()
	{
		return inserted;
	}
};

class MEMORY : public DEVICE
{
private:
	DEVICE *d_slot[4][4];
	DEVICE *d_map[4];
	bool expanded[4];
	uint8 psl;
	uint8 ssl[4];
	void change_slot();
	
public:
	MEMORY(VM* parent_vm, EMU* parent_emu) : DEVICE(parent_vm, parent_emu) {}
	~MEMORY() {}
	
	// common functions
	void reset();
	void write_data8(uint32 addr, uint32 data);
	uint32 read_data8(uint32 addr);
	uint32 fetch_op(uint32 addr, int* wait);
	void write_signal(int id, uint32 data, uint32 mask);
	
	// unique function
	void set_context_slot_expand(bool p0, bool p1, bool p2, bool p3)
	{
		expanded[0] = p0;
		expanded[1] = p1;
		expanded[2] = p2;
		expanded[3] = p3;
	}

	void set_context_slot(int p, int s, DEVICE *device)
	{
		d_slot[p][s] = device;
	}
};

class SLOT_ROMMAIN : public DEVICE
{
private:
	uint8 wdmy[0x2000];
	uint8 rdmy[0x2000];
	uint8* wbank[8];
	uint8* rbank[8];
	uint8 rom[0x8000];
	
public:
	SLOT_ROMMAIN(VM* parent_vm, EMU* parent_emu) : DEVICE(parent_vm, parent_emu) {}
	~SLOT_ROMMAIN() {}
	
	// common functions
	void initialize();
	void write_data8(uint32 addr, uint32 data);
	uint32 read_data8(uint32 addr);
};

class SLOT_ROMSUB : public DEVICE
{
private:
	uint8 wdmy[0x2000];
	uint8 rdmy[0x2000];
	uint8* wbank[8];
	uint8* rbank[8];
	uint8 rom[0x4000];
	
public:
	SLOT_ROMSUB(VM* parent_vm, EMU* parent_emu) : DEVICE(parent_vm, parent_emu) {}
	~SLOT_ROMSUB() {}
	
	// common functions
	void initialize();
	void write_data8(uint32 addr, uint32 data);
	uint32 read_data8(uint32 addr);
};

class SLOT_RAM64K : public DEVICE
{
private:
	uint8 ram[0x10000];
	
public:
	SLOT_RAM64K(VM* parent_vm, EMU* parent_emu) : DEVICE(parent_vm, parent_emu) {}
	~SLOT_RAM64K() {}
	
	// common functions
	void initialize();
	void write_data8(uint32 addr, uint32 data);
	uint32 read_data8(uint32 addr);
};

#endif

