/*
	ASCII MSX1 Emulator 'yaMSX1'
	Pioneer PX-7 Emulator 'ePX-7'
	ASCII MSX2 with DEMPA Sensor Kid Emulator 'yayaMSX2SK'

	Author : tanam
	Date   : 2013.06.29-

	modified by Takeda.Toshiya, umaiboux

	[ virtual machine ]
*/

#ifndef _MSX_H_
#define _MSX_H_

#if defined(_PX7)
#define DEVICE_NAME		"PIONEER PX-7"
#define CONFIG_NAME		"px7"
#elif defined(_MSX2SK)
#define DEVICE_NAME		"ASCII MSX2 with DEMPA Sensor Kid"
#define CONFIG_NAME		"msx2sk"
#else
#define DEVICE_NAME		"ASCII MSX1"
#define CONFIG_NAME		"msx1"
#endif

// device informations for virtual machine
#define FRAMES_PER_SEC		60
#define LINES_PER_FRAME		262
#define CPU_CLOCKS		3579545
#if defined(_PX7)
//#define SCREEN_WIDTH		256
//#define SCREEN_HEIGHT		192
#define SCREEN_WIDTH		512
#define SCREEN_HEIGHT		384
#elif defined(_MSX2SK)
#define SCREEN_WIDTH		((256 + 15)*2)	// V99X8_WIDTH
#define SCREEN_HEIGHT		((212 + 15)*2)	// V99X8_HEIGHT
#endif
#define TMS9918A_VRAM_SIZE	0x4000
#define TMS9918A_LIMIT_SPRITES
#if defined(_PX7)
#define TMS9918A_SUPER_IMPOSE
#endif
#define HAS_AY_3_8910
// for Flappy Limited '85
#define YM2203_PORT_MODE	0x80

// device informations for win32
#define USE_CART1
#define USE_CART2
#define USE_TAPE
#if defined(_PX7)
#define USE_LASER_DISC
#endif
#define USE_ALT_F10_KEY
#define USE_AUTO_KEY		6
#define USE_AUTO_KEY_RELEASE	10
#define USE_DEBUGGER

#include "../../common.h"

class EMU;
class DEVICE;
class EVENT;

class DATAREC;
class I8255;
class IO;
#if defined(_PX7)
class LD700;
#endif
class NOT;
class YM2203;
class PCM1BIT;
#if defined(_PX7)
class TMS9918A;
#elif defined(_MSX2SK)
class V99X8;
class RP5C01;
class RTCIF;
class SENSORKID;
#endif
class Z80;

class JOYSTICK;
class KEYBOARD;
class MEMORY;
class SLOT_CART;
#if defined(_PX7)
class SLOT_ROMRAM32K;
class SLOT_LDC;
#endif
#if defined(_MSX2SK)
class SLOT_ROMMAIN;
class SLOT_ROMSUB;
class SLOT_RAM64K;
#endif

class VM
{
protected:
	EMU* emu;
	
	// devices
	EVENT* event;
	
	DATAREC* drec;
	I8255* pio;
	IO* io;
#if defined(_PX7)
	LD700* ldp;
#endif
	NOT* not;
	YM2203* psg;
	PCM1BIT* pcm;
#if defined(_PX7)
	TMS9918A* vdp;
#elif defined(_MSX2SK)
	V99X8* vdp;
	RP5C01* rtc;
	RTCIF* rtcif;
	SENSORKID* sensorkid;
#endif
	Z80* cpu;
	
	JOYSTICK* joystick;
	KEYBOARD* keyboard;
	MEMORY* memory;
#if defined(_MSX2SK)
	SLOT_ROMMAIN *slotmain;
	SLOT_CART *cart[2];
	DEVICE *slot_empty;
	SLOT_ROMSUB *slotsub;
	SLOT_RAM64K *slotram;
#else
	SLOT_ROMRAM32K *slot0;
	SLOT_LDC *slot2;
	SLOT_CART *cart[2];
#endif
	
public:
	// ----------------------------------------
	// initialize
	// ----------------------------------------
	
	VM(EMU* parent_emu);
	~VM();
	
	// ----------------------------------------
	// for emulation class
	// ----------------------------------------
	
	// drive virtual machine
	void reset();
	void run();
	
#ifdef USE_DEBUGGER
	// debugger
	DEVICE *get_cpu(int index);
#endif
	
	// draw screen
	void draw_screen();
	
	// sound generation
	void initialize_sound(int rate, int samples);
	uint16* create_sound(int* extra_frames);
	int sound_buffer_ptr();
#if defined(_PX7)
	void movie_sound_callback(uint8 *buffer, long size);
#endif
	
	// user interface
	void open_cart(int drv, _TCHAR* file_path);
	void close_cart(int drv);
	bool cart_inserted(int drv);
	void play_tape(_TCHAR* file_path);
	void rec_tape(_TCHAR* file_path);
	void close_tape();
	bool tape_inserted();
#if defined(_PX7)
	void open_laser_disc(_TCHAR* file_path);
	void close_laser_disc();
	bool laser_disc_inserted();
#endif
	bool now_skip();
	
	void update_config();
	
	// ----------------------------------------
	// for each device
	// ----------------------------------------
	
	// devices
	DEVICE* get_device(int id);
	DEVICE* dummy;
	DEVICE* first_device;
	DEVICE* last_device;
};

#endif
